#include "gtest/gtest.h"  // must come first - uses T

#include <nausparse.h>

class NautyTests : public ::testing::Test {

protected:

    size_t vertices[10] {
            0, 2, 4, 6, 8, 10, 12, 14, 16, 18
    };

    int degrees[10] = {
            2, 2, 2, 2, 2, 2, 2, 2, 2, 2
    };

    int edges[20] {
            9, 1, 0, 2, 1, 3, 2, 4, 3, 5,
            4, 6, 5, 7, 6, 8, 7, 9, 8, 0
    };

    DEFAULTOPTIONS_SPARSEGRAPH(options);

    sparsegraph decagon = {
        20, vertices, 10,
        degrees, edges,
        nullptr,
        10,10,20, 0
    };

    statsblk stats {0, };

};

TEST_F(NautyTests, GroupOfDecagon) {

    int lab[10] = {0, }; // not used
    int ptn[10] = {0, };
    int orbits[10] = {0, };

    sparsenauty(&decagon, lab, ptn, orbits, &options, &stats, nullptr);

    // group size should be 20
    EXPECT_DOUBLE_EQ(stats.grpsize1, 20);
    EXPECT_EQ(stats.grpsize2, 0);
}

TEST_F(NautyTests, OrbitsOfColouredDecagon) {

    int lab[10] { 0, 5, 1,2,3,4, 6,7,8,9};
    int ptn[10] = {1, 0, 1, 1, 1, 1, 1, 1, 1, 0};
    int orbits[10];
    options.defaultptn = false; // use colours

    sparsenauty(&decagon, lab, ptn, orbits, &options, &stats, nullptr);

    // group size should be 4
    EXPECT_DOUBLE_EQ(stats.grpsize1, 4);
    EXPECT_EQ(stats.grpsize2, 0);

    int expectedOrbits[10] = {0, 1, 2, 2, 1, 0, 1, 2, 2, 1};
    for (int i=0; i < 10; i++) {
        EXPECT_EQ(orbits[i], expectedOrbits[i]) << "Arrays differ at index " << i;
    }

}

TEST_F(NautyTests, CanonicalFormDecagon) {


    int lab[10] = {0, }; // not used
    int ptn[10] = {0, };
    int orbits[10] = {0, };

    options.getcanon = true;

    SG_DECL(result); // sparse canonically labeled graph

    sparsenauty(&decagon, lab, ptn, orbits, &options, &stats, &result);

    // decagon         0 - 1 - 2 - 3 - 4 - 5 - 6 - 7 - 8 - 9 - 0
    // canonical form  0 - 8 - 7 - 4 - 3 - 1 - 2 - 5 - 6 - 9 - 0

    int expectedPerm[10] = { 0, 5, 6, 4, 3, 7, 8, 2, 1, 9 }; // maps canonical back to original

    for (int i=0; i < 10; i++) {
        EXPECT_EQ(expectedPerm[i], lab[i]) << "Wrong back image of " << i;
    }

    SG_FREE(result);
}

/**
 * Make sure that nauty is the correct version
 */
TEST_F(NautyTests, NautyVersion26r12Regression) {

    EXPECT_EQ(NAUTYVERSIONID, 26040);

    sparsegraph sgr;
    EXPECT_LT((void *)(&sgr.nde), (void*)(&sgr.v));
    EXPECT_LT((void *)(&sgr.v), (void*)(&sgr.nv));
    EXPECT_LT((void *)(&sgr.nv), (void*)(&sgr.d));
    EXPECT_LT((void *)(&sgr.d), (void*)(&sgr.e));
    EXPECT_LT((void *)(&sgr.e), (void*)(&sgr.w));
    EXPECT_LT((void *)(&sgr.w), (void*)(&sgr.vlen));
    EXPECT_LT((void *)(&sgr.vlen), (void*)(&sgr.dlen));
    EXPECT_LT((void *)(&sgr.dlen), (void*)(&sgr.elen));
    EXPECT_LT((void *)(&sgr.elen), (void*)(&sgr.wlen));


}
