#include "gtest/gtest.h"

// for now:
// #define private public

#include "../plane/Projectivity.hpp"

#define ORDER 11
using FF = FFel<ORDER>;
using Proj = Projectivity<ORDER>;
using Pt = Point<ORDER>;
constexpr int SIZE = ORDER * ORDER + ORDER + 1;

// fixture
class ProjectivityTest : public ::testing::Test {
protected:
    const Pt &p0 = Pt::get(SIZE / 5);
    const Pt &p1 = Pt::get(2 * SIZE / 5);
    const Pt &p2 = Pt::get(3 * SIZE / 5);
    const Pt &p3 = Pt::get(4 * SIZE / 7);

    Proj sigma{p0, p1, p2, p3};

};

TEST_F (ProjectivityTest, mulinverse) {
    EXPECT_TRUE ((sigma * sigma.inv()).isIdentity());
    EXPECT_TRUE ((sigma.inv() * sigma).isIdentity());
}

TEST_F (ProjectivityTest, fourpoints) {
    // 4 'random' points
    EXPECT_EQ(sigma(Pt::get(0)), p0);
    EXPECT_EQ(sigma(Pt::get(1)), p1);
    EXPECT_EQ(sigma(Pt(FF(1), FF(0), FF(0))), p2);
    EXPECT_EQ(sigma(Pt(FF(1), FF(1), FF(1))), p3);
}

TEST_F(ProjectivityTest, toBase) {
    const Proj &tau = Proj::toBase(p0, p1, p2, p3);
    EXPECT_EQ(tau(p0), Pt::get(0));
    EXPECT_EQ(tau(p1), Pt::get(1));
    EXPECT_EQ(tau(p2), Pt(FF(1), FF(0), FF(0)));
    EXPECT_EQ(tau(p3), Pt(FF(1), FF(1), FF(1)));
}