#include <SparseGraph.hpp>
#include "gtest/gtest.h"  // must come first - uses T

#include <naututil.h>
#include <nausparse.h>

class SparseGraphTests : public :: testing::Test {

protected:

    int d2[3] {0, 1, 3};

    SparseGraph sg{ 2, d2}; // Fano plane

};

TEST_F (SparseGraphTests, AutomorphismGroupWithDual) {

    int lab[14] = {0, }; // not used
    int ptn[14] = {0, };
    int orbits[14] = {0, };

    DEFAULTOPTIONS_SPARSEGRAPH(options);
    statsblk stats {0, };

    sparsenauty(&sg, lab, ptn, orbits, &options, &stats, nullptr);

    // group size should be 336
    EXPECT_DOUBLE_EQ(stats.grpsize1, 336);
    EXPECT_EQ(stats.grpsize2, 0);

}

TEST_F (SparseGraphTests, AutomorphismGroupOfPoints) {

    int lab[14] = {0, };
    int ptn[14] = {0, };
    int orbits[14] = {0, }; // not used


    DEFAULTOPTIONS_SPARSEGRAPH(options);
    options.defaultptn = false; // use colours
    // distinguish between points and lines by colouring
    for (int i=0; i < 14; i++) {
        lab[i] = i;
        ptn[i] = 1;
    }
    ptn[6] = 0; // first partition cell 0..6
    ptn[13] = 0; // second partition cell 7..13

    statsblk stats {0, };

    sparsenauty(&sg, lab, ptn, orbits, &options, &stats, nullptr);

    // group size should be 168
    EXPECT_DOUBLE_EQ(stats.grpsize1, 168);
    EXPECT_EQ(stats.grpsize2, 0);

}

TEST_F (SparseGraphTests, AutomorphismGroupOfTriangle) {

    int lab[14] = {0, };
    int ptn[14] = {0, };
    int orbits[14] = {0, }; // not used


    DEFAULTOPTIONS_SPARSEGRAPH(options);
    options.defaultptn = false; // use colours
    // distinguish between points and lines by colouring
    for (int i=0; i < 14; i++) {
        lab[i] = i;
        ptn[i] = 1;
    }
    ptn[2] = 0; // first part 0..2 (triangle)
    ptn[6] = 0; // second part 3..6 (other points)
    ptn[13] = 0; // third part 7..13 (all lines)

    statsblk stats {0, };

    sparsenauty(&sg, lab, ptn, orbits, &options, &stats, nullptr);

    // group size should be 6
    EXPECT_DOUBLE_EQ(stats.grpsize1, 6);
    EXPECT_EQ(stats.grpsize2, 0);

}