Generators for blocking sets with non-trivial automorphism group
==

This project provides generators for blocking sets with non-trivial
automorphism groups in small projective planes, in particular, the plane
of order 11.

There are several independent generators, each assuming the existence
of a particular type of *projective* automorphism (of prime
order). Together they cover all elements of prime order in the
projective automorphism group of PG(2,11), and hence all blocking sets
of PG(2,11) with a non-trivial projective automorphism group.

The programs are written in C++ and were developed using the CLion IDE
which uses `cmake` as build tool and `clang` as compiler. (It should
be possible to rebuild the software outside of CLion, as long as you
have `cmake` and `clang` available.)

A few unit tests are provided with the source code - see below - but
they are not needed to be able to run the main program.

#### Scripts
The following scripts in the `scripts` directory form the basis of the generation
of all blocking sets in the plane of order 11 with a non-trivial automorphism group,
assuming an automorphism of order 2,3,5,7,11 and 19 respectively: `invol11.sh`, `three11.sh`,
`order11case5.sh`, `order11case7.sh`, `order11case11.sh` and `order11case19.sh`.

Note that the generated sets can still contain isomorphic copies. To obtain the final result
you need to first convert all output to the same format (see [representations](formats.md)), apply
the program `canonic` to convert the sets to canonical form and then use Unix tools like `uniq` and `sort` to
remove duplicate copies.

Notes:
* The order of the field being used is compiled into the program. The order must be configured
  in `CMakeLists` (line 10)
* Some of the programs can also be run for other (lower) field orders
* The program `gensinger` generates *all* blocking sets up to equivalence, including those with a trivial group. This
is essentially the same program as used in the earlier papers on blocking sets for Desarguesian planes of order <= 9.

#### Depends on Nauty

To be able to compile and run this program, download, install and 'make' a 
[Nauty distribution](https://pallini.di.uniroma1.it/) (currently we are using version `nauty26r5` ). 
Make a softlink to `nauty` in the base directory of this project.

#### Unit tests

To build and execute the unit tests you need a directory `Google_tests/lib` which is not included
in the repository. 
Clone Google Test from the [official repository](https://github.com/google/googletest). 
Link that folder (or make a copy) to `Google_tests/lib`.
 
