#include <iostream>
#include <sstream>
#include <cstring>
#include "plane/Weights.hpp"
#include "plane/CanonicalForm.hpp"
//#include "plane/CanonicalFormX.hpp"

/**
 * Converts point sets in the projective plane to their canonic form
 */
int main(int , char** argv) {

    bool outputSize {false};
    bool outputGroupSize {false};
    bool outputSet {true};

    for (++argv; *argv; ++argv) {
        if (strcmp(*argv,"-s")==0) {
            outputSize = true;
        } else if (strcmp(*argv,"-g")==0) {
            outputGroupSize = true;
        } else if (strcmp(*argv,"-n")==0) {
            outputSet = false;
        } else {
            fprintf(stderr, "*** Warning: unknown command line option '%s' ignored.\n", *argv);
        }
    }

    constexpr int Q = ORDER_OF_FIELD;

    // input from std::cin - output to std::cout
    std::string line;
    while (std::getline(std::cin, line)) {
        std::istringstream iss(line);
        Weights<Q> weights { ProjectivePlane<Q>() };
        while (iss.peek() != EOF) {
            char ch;
            int pt;
            iss >> ch >> pt; // ch will always be -
            weights.add(pt);
        }
        weights.computeAllInvariants();
        CanonicalForm<Q> canonicalForm(weights);
//        CanonicalFormX<Q> canonicalForm(weights);
//        canonicalForm.printInvariants();
        canonicalForm.compute();
        if (outputSize) {
            printf ("%d ", weights.getSet().getSize());
        }
        if (outputGroupSize) {
            printf("%d ", canonicalForm.getGroupSize());
        }
        if (outputSet) {
            canonicalForm.getCanonicalSet().print('-');
        }
        putchar('\n');
    }

}