Representations of points and lines
===

The various programs/cases use different internal representations of
points and lines:

#### Projective coordinates (`plane`)

Field elements are represented by integers 0…10 module 11.

Points have normalized coordinates of the form (0,0,1), (0,1,*) or (1,*,*). From this representation we obtain a
*sequence number* for each point, as follows
* (0,0,1) has sequence number 0
* (0,1,z) has sequence number 1 + z
* (1,y,z) has sequence number 12 + 11y + z
(See also `plane/Point.hpp`)

Sets of points are represented as strings of sequence numbers separated and preceeded by dashes (`-`). 

#### Singer cycle exponent (`singer`)

Points are represented as integers 0…132. The point with index i is the image of (1,0,0) through σ^i, where σ is a Singer
cycle of the projective plane. (See also `singer/Singer.hpp`)

Sets of points are represented as strings of such indices separated by percent signs (`%`)

#### Semi-affine representation (`semi`)

There are three types of points
* The origin, represented as `+0`
* The points at infinity, represented as `-0`…`-10`
* The 'fat' points of the semi-affine plane, represented as `:0`…`:59`. (These represent a *pair* of points.)

The fat point with index i is the image of (1,1,0) through ρ^i and ρ^(60+i), where ρ is a 'Singer'
cycle of the semi-affine plane.
The point at infinity with index i is the image of (0,1,1) through ρ^i. (See also `semi/Semi.hpp`)

Sets of points are strings concatenating these representations.

#### Conversion programs

The following programs can be used to convert lists of sets between
formats: `planetosemi`, `planetosinger`, `semitoplane`, `singertoplane`.
