#include <chrono>
#include <sstream>
#include "plane/GeneratorFromPartition.hpp"

/**
 * Generates blocking sets in the projective plane as unions of parts of a partition. The partition is read
 * from stdin. No isomorphism reduction is done.
 */
int main() {

    constexpr int Q = ORDER_OF_FIELD;
    const int V = Q * Q + Q + 1;

    // registering clocks
    std::clock_t startcputime = std::clock();
    auto wcts = std::chrono::system_clock::now();

    Partition<V> partition;
    // read partition from stdin

    std::string line;
    while (std::getline(std::cin, line)) {
        std::istringstream iss(line);
        int part = partition.addPart();
        while (iss.peek() != EOF) {
            char ch;
            int pt;
            iss >> ch >> pt; // ch will always be -
            partition.add(part, pt);
        }
    }

    GeneratorFromPartition<Q> generator{partition};
    generator.gen();

    // showing time used
/*
    double cpu_duration = (double) (std::clock() - startcputime) / CLOCKS_PER_SEC;
    std::cerr << "Finished in " << cpu_duration << " seconds [CPU Clock] " << std::endl;

    std::chrono::duration<double> wctduration = (std::chrono::system_clock::now() - wcts);
    std::cerr << "Finished in " << wctduration.count() << " seconds [Wall Clock]" << std::endl;
*/

    return 0;

}