#include <iostream>
#include <chrono>

#include "InvolGenerator.hpp"

/**
 * Generates, up to isomorphism, all blocking sets that are fixed by the involution (x,y,z) → (x,y,-z), using the cyclic semi-affine representation
 * of the plane. Arguments are points to be included at infinity, with an additional + if also the origin must be included
 */
int main(int argc, char *argv[]) {

    // registering clocks
    std::clock_t startcputime = std::clock();
    auto wcts = std::chrono::system_clock::now();

    InvolGenerator<ORDER_OF_FIELD> generator;
    generator.processArgs(argc, argv);
    generator.generate(); // cannot be called again during same main!

    // print statistics
    generator.printStatistics();

    // showing time used
    double cpu_duration = (double) (std::clock() - startcputime) / CLOCKS_PER_SEC;
    std::cerr << "Finished in " << cpu_duration << " seconds [CPU Clock] " << std::endl;

    std::chrono::duration<double> wctduration = (std::chrono::system_clock::now() - wcts);
    std::cerr << "Finished in " << wctduration.count() << " seconds [Wall Clock]" << std::endl;

    return 0;
}

