#ifndef BSETS_INVOL_POINTSET_H
#define BSETS_INVOL_POINTSET_H

#include <array>
#include "PointSet.hpp"
#include "Invol.hpp"

/////////////////////////////////////////
// DECLARATIONS
////////////////////////////////////////

/**
 * Set of involution orbits, i.e., fat or thin points
 */
template<int Q>
class InvolSet : public PointSet<Invol<Q>::V> {

private:
    static constexpr int F = Invol<Q>::F;
    static constexpr int K = Invol<Q>::K;
    static constexpr int V = Invol<Q>::V;

public:
    using PointSet<V>::add;
    using PointSet<V>::remove;
    using PointSet<V>::getSize;
    using PointSet<V>::contains;

public:
    void addInfty(int pt) { add(F + pt); }

    void addOrigin() { add(F + K); }

    void removeInfty(int pt) { remove(F + pt); };

    void removeOrigin() { remove(F + K); };

    bool containsInfty(int pt) const { return contains(F + pt); }

    bool containsOrigin() const { return contains(F + K); }

    // size of the set, seen as a subset of PG(2,Q).
    int getSizePG() const;

    // print a representation of this set to stdout
    void print(char prefix) const;

};

/////////////////////////////////////////
// DEFINITIONS
////////////////////////////////////////

// TODO not used?
template<int Q>
void InvolSet<Q>::print(char prefix) const {
    for (int pt = 0; pt < V; pt++) {
        if (contains(pt)) {
            printf("%d-", pt);
        }
    }
}

template<int Q>
int InvolSet<Q>::getSizePG() const {
    int s = getSize();
    for (int pt = 0; pt < F; ++pt) {
        if (contains(pt)) {
            s++;
        }
    }
    return s;
}


#endif //BSETS_INVOL_POINTSET_H
