#ifndef BSETS_LINE_POINT_HPP
#define BSETS_LINE_POINT_HPP

#include "../lib/FFel.hpp"

// Note: this class is very similar to ../lib/Point.hpp but we see currently not much advantage
// in merging both classes using templates for instance

template<int Q>
class Projectivity;

////////////////////////////////////////
// DECLARATIONS
////////////////////////////////////////

/**
 * Point of PG(1,Q).
 */
template<int Q>
class Point {

    using FF = FFel<Q>;

public:
    FF x;
    FF y;

    /**
     * Point with given coordinates, not necessarily normalized
     */
    Point(const FF &x, const FF &y) : x(x), y(y) {}

    /**
     * 'point' with all zero coordinates, needed to initialize arrays. Avoid using this!
     */
    Point() : x(FF(0)), y(FF(0)) {}

    static const Point &get(int index) {
        return all()[index];
    }

    /**
     * Return the sequence number of the point. The point need not be normalized.
     * <ul>
     * <li> (0,1) → 0
     * <li> (1,*) → 1..Q+1
     * </ul>
     */
    int index() const;

    static constexpr int K = Q + 1;

    /**
     * All points of this line (normalized), ordered by index
     */
    static const std::array<Point<Q>, K> &all() {
        static std::array<Point<Q>, K> result = initAll();
        return result;
    }

    /**
     * Apply a projectivity to this point
     */
   Point<Q> operator*(const Projectivity<Q> &projectivity) const;

    /**
     * Equality of points. Arguments need not be normalized. (Everything is equal to (0,0), but this should not be used.)
     */
    bool operator==(const Point<Q> &other) const {
        return x * other.y == y * other.x;
    };

    bool operator!=(const Point<Q> &other) const {
        return x * other.y != y * other.x;
    };


private:

    static std::array<Point<Q>, K> initAll();

};

////////////////////////////////////////
// DEFINITIONS
////////////////////////////////////////

template<int Q>
int Point<Q>::index() const {
    if (x.isZero()) {
        return 0;
    } else if (x.isOne()) { // no need to normalize
        return y.index() + 1;
    } else {
        return (y / x).index() + 1;
    }
}

template<int Q>
std::array<Point<Q>, Point<Q>::K> Point<Q>::initAll() {
    std::array<Point<Q>, K> result{};

    int pos = 0;

    result[pos++] = Point{FF(0), FF(1)};

    for (const auto &element: FF::all()) {
        result[pos++] = Point{FF(1), element};
    }
    return result;
}

#endif //BSETS_LINE_POINT_HPP
