#include <array>

#include "line/Projectivity.hpp"
#include "lib/PointSet.hpp"

/*
 * Generates all subsets of the line, up to isomorphism. Uses a very straight-forward algorithm, essentially
 * running over all sets, computing all images of that set and marking them as already encountered
 */
template <int Q>
class Generator {

private:
    static constexpr int K = Q + 1;

    std::array<bool, 1 << K> marked{};

    int index{0};
    int pt{0};
    PointSet<K> currentSet{};

    // convert a point set to the corresponding index
    static int getIndex(const PointSet<K> &set);

    // mark all images of the current set
    void markImages();

    void gen() ;

public:
    void generate () {
        gen();
    }


};

int main(int , char *[]) {
    Generator<ORDER_OF_FIELD>().generate();
    return 0;
}

template<int Q>
void Generator<Q>::gen() {
    // first with current point, then without
    if (pt < K) {
        currentSet.add(pt);
        pt ++;
        gen();
        pt --;
        currentSet.remove(pt);
        pt ++;
        gen();
        pt --;
    } else {
        if (!marked[index]) {
            markImages();
            currentSet.print('-');
            std::cout << std::endl;
        }
        index ++;
    }
}

template<int Q>
int Generator<Q>::getIndex(const PointSet<K> &set) {
    int result{0};
    for (int pt=0; pt < K; pt++) {
        result *= 2;
        if (! set.contains(pt)) {
            result ++;
        }
    }
    return result;
}

template<int Q>
void Generator<Q>::markImages() {
    // all projectivities - could be done more efficiently, but no need for current application
    for (int i = 0; i < (Q-1)*Q*(Q+1); ++i) {
        Projectivity<Q> projectivity = Projectivity<Q>::withIndex(i);
        PointSet<Q + 1> image = projectivity(currentSet);
        marked[getIndex(image)] = true;
    }
}


