#ifndef BSETS_GENERATORFROMPARTITION_HPP
#define BSETS_GENERATORFROMPARTITION_HPP

#include "Partition.hpp"
#include "Weights.hpp"

/**
 * Generates all minimal blocking sets that are unions of parts of a given partition
 * of points in the projective plane. Does *not* do any isomorphism reduction so should
 * only be used for small parameters.
 */
template<int Q>
class GeneratorFromPartition {

    // Note: has quite a bit of code that is similar to GeneratorBase but it did not seem worth it
    // to extract common code. This code code also have been written more generically, like GeneratorBase.

private:
    static constexpr int V = Q*Q + Q + 1;

    Partition<V> &partition;
    Weights<Q> weights;

    int maxPart;

public:
    GeneratorFromPartition(Partition<V> &partition) :
        partition{partition}, maxPart(partition.size()), weights{ProjectivePlane<Q>()} {
    }

    // recursively generate all blocking sets using parts with index < maxPart
    void gen();

private:

    // is this a valid subset of a possible result?
    bool isValid() const {
        return weights.isMinimal();
    }

    // is the currently generated set complete?
    bool isComplete() const {
        return weights.isBlocking();
    }

    // print the set to stdout and return the number of points it contains
    int shipSet();

    // add a part of the partition to the current set
    void addPart(int part) {
        for (int pt : partition[part]) {
            weights.add(pt);
        }
    }

    // remove a part of the partition from the current set
    void removePart(int part) {
        for (int pt : partition[part]) {
            weights.remove(pt);
        }
    }

};

template<int Q>
void GeneratorFromPartition<Q>::gen() {
    if (isValid()) {
        if (isComplete()) {
            shipSet();
        } else if (maxPart > 0){
            maxPart--;
            gen();
            addPart(maxPart);
            gen();
            removePart(maxPart);
            maxPart++;
        }
    }
}

template<int Q>
int GeneratorFromPartition<Q>::shipSet() {
    // Copied from singer/Generator
    // print to stdout
    const PointSet<V> &set = this->weights.getSet();
    set.print('-');
    printf("\n");
    return set.getSize();
}

#endif //BSETS_GENERATORFROMPARTITION_HPP
