#include <string>
#include <iostream>
#include "FFel.hpp"
#include "plane/Projectivity.hpp"

/**
 * Computes and prints the orbit partition of a projectivity given as a 3 x 3 matrix
 */

constexpr int Q = ORDER_OF_FIELD;

int main(int argc, char **argv) {
    if (argc != 10) {
        std::cerr << "Usage: " << argv[0] << " <matrix element 1> ... <matrix element 9>" << std::endl;
        return 1;
    }

    FFel<Q> ffargs[9];
    // convert arguments to subsequent matrix elements
    for (int i = 0; i < 9; ++i) {
        ffargs[i] = FFel<Q>::get(std::stoi(argv[i + 1]));
    }
    Projectivity<Q> proj{Projectivity<Q>::Matrix {
            ffargs[0], ffargs[1], ffargs[2],
            ffargs[3], ffargs[4], ffargs[5],
            ffargs[6], ffargs[7], ffargs[8]
    }};
    Partition partition = proj.orbitPartition();
    if (!partition.isPartition()) {
        std::cerr << "Error: not a partition" << std::endl;
        return 1;
    } else {
        partition.print('-');
        return 0;
    }


}