#ifndef BSETS_SINGER_GENERATOR_HPP
#define BSETS_SINGER_GENERATOR_HPP

#include <array>
#include "Singer.hpp"
#include "Weights.hpp"
#include "ColourablePlane.hpp"
#include "Archive.hpp"
#include "SingerGeometry.hpp"
#include "GeneratorBase.hpp"

/////////////////////////////////////////
// DECLARATIONS
////////////////////////////////////////

template<int Q>
class Generator : public GeneratorBase<Weights<Q>,Singer<Q>::V, Singer<Q>::V, Singer<Q>::V>{

private:

    static constexpr int V = Singer<Q>::V;

    using GeneratorBase<Weights<Q>,V,V,V>::add;
    using GeneratorBase<Weights<Q>,V,V,V>::remove;
    using GeneratorBase<Weights<Q>,V,V,V>::gen;

protected:
    virtual int shipSet(); // implements

public:

    Generator() : GeneratorBase<Weights<Q>,V,V,V>(SingerGeometry<Q>()) {

    }

public:
    void generate();

    void processArgs(int argc, char *argv[]);

};

/////////////////////////////////////////
// DEFINITIONS
////////////////////////////////////////

template<int Q>
void Generator<Q>::generate() {

    // start with a two point subset. All two point subsets are equivalent/isomorphic
    add(0);
    add(1);

    gen();

    remove(1);
    remove(0); // these were added to be able to check memory leaks

}

template<int Q>
void Generator<Q>::processArgs(int argc, char **argv) {
    if (argc > 1) {
        if (argc != 4) {
            std::cerr << "Usage: " << argv[0] << " depth index modulus";
        } else {
            this->splitDepth = (int) strtol(argv[1], nullptr, 10);
            this->splitIndex = (int) strtol(argv[2], nullptr, 10);
            this->splitModulus = (int) strtol(argv[3], nullptr, 10);
        }
    }
}

template<int Q>
int Generator<Q>::shipSet() {
    // print to stdout
    const PointSet<V> &set = this->weights.getSet();
    set.print('%');
    printf("\n");
    return set.getSize();
}




#endif //BSETS_SINGER_GENERATOR_HPP
