#ifndef BSETSNONTRIV_THREESET_HPP
#define BSETSNONTRIV_THREESET_HPP

#include <array>
#include "PointSet.hpp"
#include "Three.hpp"

/////////////////////////////////////////
// DECLARATIONS
////////////////////////////////////////

/**
 * Set of orbits under automorphism of order 3, i.e., fat or thin points
 */
template<int Q>
class ThreeSet : public PointSet<Three<Q>::V> {

private:
    static constexpr int F = Three<Q>::F;
    static constexpr int K = Three<Q>::K;
    static constexpr int V = Three<Q>::V;

public:
    using PointSet<V>::add;
    using PointSet<V>::remove;
    using PointSet<V>::getSize;
    using PointSet<V>::contains;

public: //TODO F+K = V-1 but maybe some continuity in notation is nice.
    void addInfty(int pt) { add(F + pt); }

    void addOrigin() { add(F + K); }

    void removeInfty(int pt) { remove(F + pt); };

    void removeOrigin() { remove(F + K); };

    bool containsInfty(int pt) const { return contains(F + pt); }

    bool containsOrigin() const { return contains(F + K); }

    // size of the set, seen as a subset of PG(2,Q).
    int getSizePG() const;

    // print a representation of this set to stdout
    void print(char prefix) const;

};

/////////////////////////////////////////
// DEFINITIONS
////////////////////////////////////////

// TODO not used?
template<int Q>
void ThreeSet<Q>::print(char prefix) const {
    for (int pt = 0; pt < V; pt++) {
        if (contains(pt)) {
            printf("%d-", pt);
        }
    }
}

template<int Q>
int ThreeSet<Q>::getSizePG() const {
    int s = getSize();
    for (int pt = 0; pt < F; ++pt) {
        if (contains(pt)) {
            s++;
        }
    }
    return s;
}

#endif //BSETSNONTRIV_THREESET_HPP
